from bs4 import BeautifulSoup 
import requests
import json
import re
from .utils import getDepdecs,getDepTasks,getDepName,getDepAgenda,getQuestPage,getDepQuest,getQuestDt

def getUrls():
    '''
    Get deputy urls for scrapping
    
    '''
    pages=['']+['='.join(["?page",str(i)]) for i in range(1,33)]
    url_prfx="https://www.chambredesrepresentants.ma/fr/%D8%AF%D9%84%D9%8A%D9%84-%D8%A3%D8%B9%D8%B6%D8%A7%D8%A1-%D9%85%D8%AC%D9%84%D8%B3-%D8%A7%D9%84%D9%86%D9%88%D8%A7%D8%A8/2021-2026"
    urls=dict()
    for page in pages:
        url=''.join([url_prfx,page])
        req = requests.get(url)
        soup = BeautifulSoup(req.content, "html.parser")
        divs=soup.find_all('div',class_="woman-slider-content")
        for div in divs:
            span=div.find_all('span')
            dep_name=span[0].text.strip()
            dep_url=span[0].find('a',href=True)["href"]
            urls[dep_name]=''.join(["https://www.chambredesrepresentants.ma",dep_url])
    return urls

def getDeputy(url,include_quest=True,quest_link=False,quest_det=False):
    '''
    > Scrap the web page of a deputy
    >params :
           - url : link to the deputy link
           - include_quest (default=True): True to retuen questions
           - quest_link (default=False) True to include links of each question
           - quest_det(default=False) = True to include question details
    return a dictionary :
          {"Nom":name,"description":desc, "task":task,"Agenda":agenda,"Questions":Quests}
    '''
    req = requests.get(url)
    soup = BeautifulSoup(req.content, "html.parser")
    ## Deputy name
    name=getDepName(soup)
    ## Description
    desc=getDepdecs(soup)
    ##Parliamentary tasks
    task=getDepTasks(soup)
    ## Deputy agenda
    agenda=getDepAgenda(soup)
    ## Deputy Quest
    Quests=[]
    if include_quest==True:
        Quests=getDepQuest(url,quest_link,quest_det)
    return {"Nom":name,"description":desc, "task":task,"Agenda":agenda,"Questions":Quests}


import pdfplumber
from bidi.algorithm import get_display
import arabic_reshaper
import re
import json
import os
from .utils import storeJSON,storeTXT,createDir,storeAllJSON,storeAllTXT

def getRawTrscp(path,start=1,end=-1):
    '''
    > Get the raw text of the transcript
    > It returns a dictionary of dictionaries in the form of :
       {'page_1':{"page":page,"rigth":right,"left":left}}
       where : - page_1: page number (e.g.page_x id page number x)
               - page : the text of page_1
               - right : the text of the right side of page_1
               - left : the text of the left side of page_1
    > params :
        - path: pdf path
        - start (default =1) : the number of the page from where you want to begin extracting text.
                               The first page is 1 not 0!
        - end (default=-1) : the number of the page to where you want to stop extracting text
    
    '''
    with pdfplumber.open(path) as pdf:
        x0 = 0    # Distance of left side of character from left side of page.
        x1 = 0.5  # Distance of right side of character from left side of page.
        y0 = 0  # Distance of bottom of character from bottom of page.
        y1 = 0.9  # Distance of top of character from bottom of page.
        text=dict()
        if end==-1:
            end=len(pdf.pages)
        for pgNbr in range(start-1,end):
            page = pdf.pages[pgNbr]
            left=page.crop((x0*float(page.width),y0*float(page.height),x1*float(page.width),y1*float(page.height)))
            page_text_left=left.extract_text()
            bidi_text_left = get_display(page_text_left)
            reshaped_text_left = arabic_reshaper.reshape(bidi_text_left)
            right=page.crop((0.5*float(page.width), y0*float(page.height), 1*float(page.width), y1*float(page.height)))
            page_text_right=right.extract_text()
            bidi_text_right = get_display(page_text_right)
            reshaped_text_right = arabic_reshaper.reshape(bidi_text_right)
            page_text = '\n'.join([reshaped_text_right, reshaped_text_left])
            text['_'.join(['page',str(pgNbr+1)])]={
                "page":page_text,
                "right":reshaped_text_right,
                "left":reshaped_text_left}
    return text



def texToDir(path_pdf,parent_dir,start=1,end=-1,both_format=True,file_type='json'):
    '''
    > Create a directory to store files 
    > params :
        - path_pdf : pdf path
        - parent_dir : directory where to create a directory to store data
        - start (default=1): Number of the page from where you want to begin extracting text.
                               The first page is 1 not 0!
        - end (default=-1) : Number of the page to where you want to stop extracting text
        - both_format (default=True) :True if you want to store file in json AND txt format.
                                   If it's False, you may sepecify format with file_type
        - file_type (default ='json') : json or txt. It both_format= True, you don't need to use this parameter
    '''
    text=getRawTrscp(path_pdf,start,end)
    print("text parsing completed")
    file_name=path_pdf.split('/')[-1].replace('.pdf','')
    path_dir=createDir(parent_dir,file_name)
    print("Directory ceated")
    if both_format==True:
        storeAllJSON(file_name,path_dir,text)
        print("JSON file created")
        storeAllTXT(path_dir,text,file_name)
        print("Text files created")
    else :
        if file_type=='json':
            storeAllJSON(file_name,path_dir,text)
        else :
            storeAllTXT(path_dir,text,file_name)


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="barlaman",
    version="0.0.1",
    author="ANDAM Amine",
    author_email="andamamine83@gmail.com",
    description='Scrape data of Moroccan House of Representatives.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/nowabindata/barlaman',
    keywords=["politics","parliament","webscraping","parliamentary-data","morocco"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
)
